package com.tencent.edu.common.misc

import android.animation.Animator
import android.animation.ObjectAnimator
import android.app.Activity
import android.content.Context
import android.graphics.Color
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout

import com.tencent.edu.R
import com.tencent.edu.common.compat.WindowCompat
import com.tencent.edu.utils.PixelUtil


/**
 * 沉浸式状态栏填充
 *
 * @author aidanyang
 * @date 2020-04-10
 */
object StatusBarPlaceHolder {

    // 动画时间
    private const val DURATION: Long = 250

    // 渐变动画
    private var mHideAnim: ObjectAnimator? = null
    private var mShowAnim: ObjectAnimator? = null

    private var mWeakView: WeakReference<View>? = null

    var enableStatusBarColorChanged = true

    @JvmOverloads
    fun create(context: Context, viewGroup: ViewGroup? = null, index: Int = -1): View {
        release()

        val view = View(context)

        view.setBackgroundColor(Color.WHITE)

        val height = getStatusBarHeight(context)

        val params = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, height)
        view.layoutParams = params

        mWeakView = WeakReference(view)

        // 将view加入到外部layout中，顶替状态栏
        viewGroup?.addView(view, index)

        return view
    }

    fun setAlpha(view: View, alpha: Boolean) {
        if (!checkEnv(view)) {
            return
        }

        mWeakView?.get()?.run {
            if (enableStatusBarColorChanged) {
                if (alpha) {
                    WindowCompat.setStatusBarDarkMode(context as Activity?, false)
                } else {
                    // 半透明状态栏，使用LightMode
                    WindowCompat.setStatusBarDarkMode(context as Activity?, true)
                }
            }

            setAlpha(1.0f)

            if (alpha) {
                setBackgroundResource(R.drawable.status_bar_alpha)
            } else {
                setBackgroundColor(Color.WHITE)
            }
        }
    }

    fun setBlack(view: View) {
        if (!checkEnv(view)) {
            return
        }

        mWeakView?.get()?.run {
            if (enableStatusBarColorChanged) {
                // 半透明状态栏，使用LightMode
                WindowCompat.setStatusBarDarkMode(context as Activity?, false)
            }

            alpha = 1.0f
            setBackgroundColor(Color.BLACK)
        }
    }

    fun createWithAnim(activity: Activity, viewGroup: ViewGroup): View {
        val view = create(activity, viewGroup)

        initAnim(view)

        setAlpha(view, true)

        return view
    }

    private fun initAnim(view: View) {
        initShowAnim(view)
        initHideAnim(view)
    }

    private fun initHideAnim(view: View) {
        mHideAnim = ObjectAnimator.ofFloat(view, "alpha", 1f, 0.1f).setDuration(DURATION)
        mHideAnim?.addListener(object : Animator.AnimatorListener {
            override fun onAnimationStart(animator: Animator) {}

            override fun onAnimationEnd(animator: Animator) {
                setAlpha(view, true)
            }

            override fun onAnimationCancel(animator: Animator) {}

            override fun onAnimationRepeat(animator: Animator) {}
        })
    }

    private fun initShowAnim(view: View) {
        mShowAnim = ObjectAnimator.ofFloat(view, "alpha", 0.1f, 1f).setDuration(DURATION)
        mShowAnim?.addListener(object : Animator.AnimatorListener {
            override fun onAnimationStart(animator: Animator) {
                setAlpha(view, false)
            }

            override fun onAnimationEnd(animator: Animator) {

            }

            override fun onAnimationCancel(animator: Animator) {}

            override fun onAnimationRepeat(animator: Animator) {}
        })
    }

    fun getStatusBarHeight(context: Context): Int {
        var height = 0
        if (context is Activity) {
            height = WindowCompat.getStatusBarHeight(context)
        }
        if (height <= 0) {
            height = PixelUtil.dp2px(25f, context.resources)
        }
        return height
    }

    fun startShowAnimation(view: View?) {
        if (!checkEnv(view)) {
            return
        }

        mHideAnim?.cancel()

        if (mShowAnim == null) {
            initShowAnim(view!!)
        } else {
            mShowAnim!!.cancel()
        }
        mShowAnim?.start()
    }

    fun startHideAnimation(view: View?) {
        if (!checkEnv(view)) {
            return
        }

        mShowAnim?.cancel()

        if (mHideAnim == null) {
            initHideAnim(view!!)
        } else {
            mHideAnim?.cancel()
        }
        mHideAnim?.start()
    }

    private fun checkEnv(view: View?): Boolean {
        if (view == null) {
            return false
        }

        if (view != mWeakView?.get()) {
            mWeakView = WeakReference(view)

            initAnim(view)
        }

        return true
    }

    fun release() {
        mShowAnim?.cancel()
        mShowAnim = null
        mHideAnim?.cancel()
        mHideAnim = null
        mWeakView?.clear()
        mWeakView = null
    }

}
