package com.tencent.edu.kernel.account

import com.tencent.edu.common.callback.Callback
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.framework.EduFramework
import com.tencent.edu.kernel.login.mgr.AccountMgr
import com.tencent.edu.kernel.protocol.AuthType
import com.tencent.edu.kernel.protocol.WnsRequest
import com.tencent.edu.module.mobileverify.MobileVerifyCenter
import com.tencent.edu.protocol.ICSRequestListener
import com.tencent.edu.protocol.impl.ProtocolManager
import com.tencent.mobileqq.pb.ByteStringMicro
import com.tencent.pbaccountanalysis.pbaccountanalysis
import com.tencent.pbaccountbind.pbaccountbind
import com.tencent.pbaccountlogin.pbaccountlogin

object AccountLoginOrBindRequester {

    private const val TAG = "AccountLoginOrBindRequester"

    /**
     * 切换登录
     */
    @JvmStatic
    fun switchLogin(switchTo: Long, switchToType: Int, callback: Callback<pbaccountlogin.SwitchLoginRsp>) {
        val req = pbaccountlogin.SwitchLoginReq()
        req.switch_to.set(switchTo)
        req.switch_to_type.set(switchToType)
        val oldToken = AccountMgr.getInstance().token

        val token = pbaccountlogin.TinyUserToken()
        token.qq_token_type.set(oldToken.qqTokenTypeValue)
        token.token.set(ByteStringMicro.copyFrom(oldToken.token.toByteArray()))
        token.token_type.set(oldToken.tokenTypeValue)

        req.token.set(token)
        val wnsRequest = WnsRequest(
                AuthType.WithAuth, "SwitchLogin", req, pbaccountlogin.SwitchLoginRsp::class.java)
        ProtocolManager.getInstance().execute(wnsRequest, object : ICSRequestListener<pbaccountlogin.SwitchLoginRsp> {
            override fun onReceived(bizCode: Int, bizMessage: String, rsp: pbaccountlogin.SwitchLoginRsp) {
                if (bizCode != 0) {
                    onError(bizCode, bizMessage)
                    return
                }

                callback.onSucc(rsp.get())
                LogUtils.i(TAG, "switchLogin success")
            }

            override fun onError(errorCode: Int, errorMessage: String) {
                LogUtils.e(TAG, "switchLogin errorCode=$errorCode,errorMsg=$errorMessage")
                callback.onError(errorCode, errorMessage)
            }
        }, EduFramework.getUiHandler())
    }

    /**
     * 绑定手机号
     */
    @JvmStatic
    fun bindPhoneAcc(
            mobile: String,
            nationCode: String,
            verifyCode: String,
            codeType: String,
            callback: Callback<pbaccountbind.BindAccRsp>
    ) {
        val req = pbaccountbind.BindPhoneAccReq()

        req.scene.set(MobileVerifyCenter.SCENE)
        req.platform.set("android")
        req.nation_code.set(nationCode)
        req.phone_number.set(mobile)
        req.verify_code.set(verifyCode)
        req.code_type.set(codeType)

        val wnsRequest = WnsRequest(
                AuthType.WithAuth, "BindPhone", req, pbaccountbind.BindAccRsp::class.java)
        ProtocolManager.getInstance().execute(wnsRequest, object : ICSRequestListener<pbaccountbind.BindAccRsp> {
            override fun onReceived(bizCode: Int, bizMessage: String, rsp: pbaccountbind.BindAccRsp) {
                if (bizCode != 0) {
                    onError(bizCode, bizMessage)
                    return
                }

                if (rsp.get() == null) {
                    onError(-1, "rsp == null")
                }

                callback.onSucc(rsp.get())
                LogUtils.i(TAG, "bindPhoneAcc success")
            }

            override fun onError(errorCode: Int, errorMessage: String) {
                LogUtils.e(TAG, "bindPhoneAcc errorCode=$errorCode,errorMsg=$errorMessage")
                callback.onError(errorCode, errorMessage)
            }
        }, EduFramework.getUiHandler())
    }

    /**
     * 绑定手机号
     */
    @JvmStatic
    fun bindPhoneAccOneKey(
            encryptedPhone: String,
            msgId: String,
            token: String,
            callback: Callback<pbaccountbind.BindAccRsp>
    ) {
        val req = pbaccountbind.BindOneKeyAccReq()

        req.encrypted_phone.set(encryptedPhone)
        req.msg_id.set(msgId)
        req.token.set(token)

        val wnsRequest = WnsRequest(
                AuthType.WithAuth, "BindOneKeyPhoneAcc", req, pbaccountbind.BindAccRsp::class.java)
        ProtocolManager.getInstance().execute(wnsRequest, object : ICSRequestListener<pbaccountbind.BindAccRsp> {
            override fun onReceived(bizCode: Int, bizMessage: String, rsp: pbaccountbind.BindAccRsp) {
                if (bizCode != 0) {
                    onError(bizCode, bizMessage)
                    return
                }

                if (rsp.get() == null) {
                    onError(-1, "rsp == null")
                }

                callback.onSucc(rsp.get())
                LogUtils.i(TAG, "bindPhoneAccOneKey success")
            }

            override fun onError(errorCode: Int, errorMessage: String) {
                LogUtils.e(TAG, "bindPhoneAccOneKey errorCode=$errorCode,errorMsg=$errorMessage")
                callback.onError(errorCode, errorMessage)
            }
        }, EduFramework.getUiHandler())
    }

    /**
     * 账号是否绑定手机号
     */
    @JvmStatic
    fun isPhoneBind(callback: Callback<Boolean>) {
        val req = pbaccountbind.IsPhoneBindReq()

        val wnsRequest = WnsRequest(
                AuthType.WithAuth, "IsPhoneBind", req, pbaccountbind.IsPhoneBindRsp::class.java)
        ProtocolManager.getInstance().execute(wnsRequest, object : ICSRequestListener<pbaccountbind.IsPhoneBindRsp> {
            override fun onReceived(bizCode: Int, bizMessage: String, rsp: pbaccountbind.IsPhoneBindRsp) {
                if (bizCode != 0) {
                    onError(bizCode, bizMessage)
                    return
                }

                callback.onSucc(rsp.is_bind.get())
                LogUtils.i(TAG, "isPhoneBind success")
            }

            override fun onError(errorCode: Int, errorMessage: String) {
                LogUtils.e(TAG, "isPhoneBind errorCode=$errorCode,errorMsg=$errorMessage")
                callback.onError(errorCode, errorMessage)
            }
        }, EduFramework.getUiHandler())
    }

    /**
     * 获取用户所有资产账号列表
     */
    @JvmStatic
    fun getAllAssetAccounts(callback: Callback<List<pbaccountbind.AccInfoRsp>?>) {
        val req = pbaccountbind.GetAllAssAccsReq()
        req.complete.set(0)
        val wnsRequest = WnsRequest(
                AuthType.WithAuth, "GetAllAssetAccounts", req, pbaccountbind.GetAllAssAccsRsp::class.java)
        ProtocolManager.getInstance().execute(wnsRequest, object : ICSRequestListener<pbaccountbind.GetAllAssAccsRsp> {
            override fun onReceived(bizCode: Int, bizMessage: String, rsp: pbaccountbind.GetAllAssAccsRsp) {
                if (bizCode != 0) {
                    onError(bizCode, bizMessage)
                    return
                }

                callback.onSucc(rsp.asset_accounts.get())
                LogUtils.i(TAG, "getAllAssetAccounts success")
            }

            override fun onError(errorCode: Int, errorMessage: String) {
                LogUtils.e(TAG, "getAllAssetAccounts errorCode=$errorCode,errorMsg=$errorMessage")
                callback.onError(errorCode, errorMessage)
            }
        }, EduFramework.getUiHandler())
    }

    /**
     * 获取用户信息
     */
    @JvmStatic
    fun getUserAllInfo(callback: Callback<pbaccountlogin.GetUserAllInfoRsp>) {
        val req = pbaccountlogin.GetUserAllInfoReq()

        val wnsRequest = WnsRequest(
                AuthType.WithAuth, "GetUserAllInfo", req, pbaccountlogin.GetUserAllInfoRsp::class.java)
        ProtocolManager.getInstance().execute(wnsRequest, object : ICSRequestListener<pbaccountlogin.GetUserAllInfoRsp> {
            override fun onReceived(bizCode: Int, bizMessage: String, rsp: pbaccountlogin.GetUserAllInfoRsp) {
                if (bizCode != 0) {
                    onError(bizCode, bizMessage)
                    return
                }

                callback.onSucc(rsp.get())
                LogUtils.i(TAG, "getUserAllInfo success")
            }

            override fun onError(errorCode: Int, errorMessage: String) {
                LogUtils.e(TAG, "getUserAllInfo errorCode=$errorCode,errorMsg=$errorMessage")
                callback.onError(errorCode, errorMessage)
            }
        }, EduFramework.getUiHandler())
    }

    /**
     * 是否是灰度账号
     */
    @JvmStatic
    fun isAssGray(callback: Callback<Boolean>) {
        val req = pbaccountanalysis.IsAssGrayReq()

        val wnsRequest = WnsRequest(
                AuthType.WithAuth, "IsAssGray", req, pbaccountanalysis.IsAssGrayRsp::class.java)
        ProtocolManager.getInstance().execute(wnsRequest, object : ICSRequestListener<pbaccountanalysis.IsAssGrayRsp> {
            override fun onReceived(bizCode: Int, bizMessage: String, rsp: pbaccountanalysis.IsAssGrayRsp) {
                if (bizCode != 0) {
                    onError(bizCode, bizMessage)
                    return
                }

                callback.onSucc(rsp.is_gray.get() == 1L)
                LogUtils.i(TAG, "IsAssGray success，is_gray == " + rsp.is_gray.get())
            }

            override fun onError(errorCode: Int, errorMessage: String) {
                LogUtils.e(TAG, "IsAssGray errorCode=$errorCode,errorMsg=$errorMessage")
                callback.onError(errorCode, errorMessage)
            }
        }, EduFramework.getUiHandler())
    }

}