package com.tencent.edu.kernel.http.retrofit

import com.tencent.edu.common.utils.LogUtils
import io.reactivex.exceptions.UndeliverableException
import io.reactivex.functions.Consumer
import io.reactivex.plugins.RxJavaPlugins
import okhttp3.OkHttpClient
import retrofit2.Retrofit
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory
import retrofit2.converter.gson.GsonConverterFactory
import java.io.IOException
import java.lang.reflect.Proxy.newProxyInstance
import java.util.concurrent.TimeUnit


/**
 * Retrofit管理类
 * @author aidanyang
 * @date 2020-08-11
 */
object RetrofitManager {
    private const val TAG = "RetrofitManager"
    private const val DEFAULT_CONNECT_TIME = 5
    private const val DEFAULT_WRITE_TIME = 10
    private const val DEFAULT_READ_TIME = 10

    private lateinit var okHttpClient: OkHttpClient
    private var retrofit: Retrofit

    private var cookieList = hashSetOf<String>()
    private var referList = hashSetOf<String>()

    init {
        retrofit = Retrofit.Builder()
                .client(initOKHttp()) //设置使用okhttp网络请求
                .baseUrl("https://ke.qq.com")
                .addCallAdapterFactory(RxJava2CallAdapterFactory.create()) //添加回调库，采用RxJava
                .addConverterFactory(GsonConverterFactory.create())
                .build()

        RxJavaPlugins.setErrorHandler(object : Consumer<Throwable> {
            override fun accept(e: Throwable) {
                if (e is UndeliverableException) {
                    LogUtils.d(TAG, "UndeliverableException=${e.cause}")
                    return
                } else if (e is IOException) {
                    // fine, irrelevant network problem or API that throws on cancellation
                    return
                } else if (e is InterruptedException) {
                    // fine, some blocking code was interrupted by a dispose call
                    return
                } else if (e is NullPointerException || e is IllegalArgumentException) {
                    // that's likely a bug in the application
                    Thread.currentThread().uncaughtExceptionHandler.uncaughtException(Thread.currentThread(), e)
                    return
                } else if (e is IllegalStateException) {
                    // that's a bug in RxJava or in a custom operator
                    Thread.currentThread().uncaughtExceptionHandler.uncaughtException(Thread.currentThread(), e)
                    return
                }
                LogUtils.d(TAG, "unknown exception=$e")
            }
        })
    }

    private fun initOKHttp(): OkHttpClient {
        okHttpClient = OkHttpClient.Builder()
                .addInterceptor(MyInterceptor(cookieList, referList))
                .connectTimeout(DEFAULT_CONNECT_TIME.toLong(), TimeUnit.SECONDS)
                .readTimeout(DEFAULT_READ_TIME.toLong(), TimeUnit.SECONDS)
                .writeTimeout(DEFAULT_WRITE_TIME.toLong(), TimeUnit.SECONDS)
                .retryOnConnectionFailure(true)
                .build()
        return okHttpClient
    }

    fun <T> create(service: Class<T>): T {
        val apiService = retrofit.create(service)
        return newProxyInstance(
                service.classLoader,
                arrayOf<Class<*>>(service),
                RetrofitProxy(apiService, { url: String, cookie: Boolean, refer: Boolean ->
                    if (cookie) cookieList.add(url)
                    if (refer) referList.add(url)
                })) as T
    }

}