/*
 * Copyright © 1998 - 2020 Tencent. All Rights Reserved.
 * 腾讯公司 版权所有
 */
package com.tencent.edu.module.course.task.courseware

import android.content.Context
import android.media.MediaPlayer
import android.util.Log
import android.view.MotionEvent
import android.view.View
import android.widget.FrameLayout
import android.widget.SeekBar
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleObserver
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.OnLifecycleEvent
import com.tencent.edu.R
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.utils.StringUtil
import kotlinx.android.synthetic.main.layout_preview_audio.view.*
import java.io.File
import java.util.*

/**
 * @author aidanyang
 * @date 2020-6-28
 */
class AudioPreviewView constructor(context: Context) : FrameLayout(context), LifecycleObserver {

    private var mTimer: Timer? = null
    private lateinit var mPlayer: MediaPlayer
    private var isPrepared = false
    private var isPause = false
    private var mLastTime = 0

    private val autoHideRunnable = Runnable {
        preview_ctrl.visibility = View.GONE
    }

    init {
        initView()
        initPlayer()
    }

    private fun initPlayer() {
//        mPlayer = MediaPlayer.create()
//        mPlayer.setPlayProxy(MyLocalFilePlayCache())
        mPlayer = MediaPlayer()
        mPlayer.setOnPreparedListener {
            isPrepared = true
            preview_total_time.text = StringUtil.millisToStringShort(it.duration.toLong())
            startPlay()
        }
        mPlayer.setOnCompletionListener {
            playComplete()
        }
    }

    private fun playComplete() {
        cancelAutoHideCtrl()
        preview_ctrl.setImageResource(R.drawable.ic_preview_play)
        preview_ctrl.visibility = View.VISIBLE
        isPause = true

        stopTimer()
        preview_seek_time.text = preview_total_time.text
        preview_seek_bar.progress = MAX_PROGRESS
    }

    private fun initView() {
        View.inflate(context, R.layout.layout_preview_audio, this)

        preview_seek_bar.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(seekBar: SeekBar?, progress: Int, fromUser: Boolean) {
                val msec = mPlayer.duration * progress / MAX_PROGRESS
                preview_seek_time.text = StringUtil.millisToStringShort(msec.toLong())
            }

            override fun onStartTrackingTouch(seekBar: SeekBar?) {
                stopTimer()
            }

            override fun onStopTrackingTouch(seekBar: SeekBar?) {
                val msec = mPlayer.duration * preview_seek_bar.progress / MAX_PROGRESS
                mPlayer.seekTo(msec)

                if (mPlayer.currentPosition != mPlayer.duration) {
                    startPlay()
                    autoHideCtrl()
                } else {
                    playComplete()
                }
            }
        })

        preview_ctrl.setOnClickListener {
            when {
                mPlayer.isPlaying -> {
                    isPause = true
                    pausePlay()
                    cancelAutoHideCtrl()
                }
                else -> {
                    prepareOrStartPlay()
                    autoHideCtrl()
                }
            }
        }

        preview_container.setOnTouchListener { _, event ->
            if (isPause) {
                return@setOnTouchListener true
            }

            if (event.action == MotionEvent.ACTION_DOWN) {
                if (preview_ctrl.visibility == View.GONE) {
                    preview_ctrl.visibility = View.VISIBLE
                } else {
                    preview_ctrl.visibility = View.GONE
                }

                autoHideCtrl()
            }
            true
        }
    }

    // 自动隐藏控制按钮
    private fun autoHideCtrl() {
        cancelAutoHideCtrl()
        postDelayed(autoHideRunnable, CTRL_AUTO_HIDE_DELAY)
    }

    private fun cancelAutoHideCtrl() {
        removeCallbacks(autoHideRunnable)
    }

    private fun prepareOrStartPlay() {
        if (isPrepared) {
            startPlay()
        } else {
            mPlayer.prepareAsync()
        }
    }

    private fun startPlay() {
        preview_ctrl.setImageResource(R.drawable.ic_preview_pause)

        mLastTime = 0
        isPause = false
        startTimer()
        mPlayer.start()
    }

    private fun pausePlay() {
        preview_ctrl.setImageResource(R.drawable.ic_preview_play)

        // isPause不放在这里因为考虑到要判断onResume时是否继续播放
        stopTimer()
        mPlayer.pause()
    }

    fun setFile(path: String) {
        val file = File(path)
        if (file.length() == 0L) {
            return
        }

        preview_name.text = file.name
        preview_size.text = StringUtil.getFileSizeString(file.length())
        mPlayer.setDataSource(path)
        prepareOrStartPlay()
    }

    private fun startTimer() {
        mTimer = Timer()
        mTimer?.schedule(object : TimerTask() {
            override fun run() {
                ThreadMgr.getInstance().executeOnUiThread {
                    if (isPause){
                        return@executeOnUiThread
                    }

                    // 防止播放到最后突然回退的问题
                    if (mLastTime > mPlayer.currentPosition) {
                        return@executeOnUiThread
                    }

                    mLastTime = mPlayer.currentPosition
                    preview_seek_time.text = StringUtil.millisToStringShort(mPlayer.currentPosition.toLong())
                    preview_seek_bar.progress = (mPlayer.currentPosition.toFloat() / mPlayer.duration * 100).toInt()
                }
            }
        }, TIMER_PERIOD, TIMER_PERIOD)
    }

    private fun stopTimer() {
        mTimer?.cancel()
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_RESUME)
    fun onResume(owner: LifecycleOwner) {
        Log.e(TAG, "onResume")

        if (isPrepared && !isPause) {
            prepareOrStartPlay()
        }
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_PAUSE)
    fun onPause(owner: LifecycleOwner) {
        Log.e(TAG, "onPause")

        if (isPrepared && mPlayer.isPlaying) {
            pausePlay()
        }
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    fun onDestroy(owner: LifecycleOwner) {
        Log.e(TAG, "onDestroy")

        mPlayer.stop()
        mPlayer.release()
        mTimer?.cancel()
    }

    companion object {

        private const val TAG = "AudioPreviewView"

        // 3s后隐藏控制按钮
        private const val CTRL_AUTO_HIDE_DELAY = 3000L

        private const val MAX_PROGRESS = 100

        // 更新SeekBar的时间
        private const val TIMER_PERIOD = 100L

    }
}