package com.tencent.edu.module.course.task.courseware

import android.text.TextUtils
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleObserver
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.OnLifecycleEvent
import com.google.gson.JsonObject
import com.tencent.edu.common.utils.DateUtil
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.SharedPrefsUtil
import com.tencent.edu.kernel.http.HttpModel
import com.tencent.edu.kernel.http.data.JsonDataObserver
import com.tencent.edu.kernel.http.util.GsonUtil
import io.reactivex.disposables.Disposable
import java.util.*

/**
 * @author aidanyang
 * @date 2020-05-28
 */
class CoursewareRequester : LifecycleObserver {
    private var mNoticeCall: Disposable? = null
    private var mUrlCall: Disposable? = null
    private var mCourseId: String? = null
    private var mTime: String? = null

    fun fetchCoursewareNotice(listener: INoticeListener) {
        if (mCourseId.isNullOrBlank()) {
            return
        }

        mNoticeCall?.dispose()

        HttpModel.fetchCoursewareNotice(mCourseId!!, object : JsonDataObserver() {

            override fun onSubscribe(d: Disposable) {
                mNoticeCall = d
                LogUtils.i(TAG, "fetchCoursewareNotice request start")
            }

            override fun onSuccessWithResult(result: JsonObject) {
                val time = GsonUtil.getString(result, "time")
                val num = GsonUtil.getString(result, "num")
                if (!time.isNullOrBlank() && !num.isNullOrBlank()) {
                    redUpdate(time, num)
                    val isNeedShow = redIsNeedShow()
                    listener.onResult(isNeedShow, if (isNeedShow) getUpdateTitle(time) else null)
                }
                LogUtils.i(TAG, "fetchCoursewareNotice request success")
            }

            override fun onError(code:Int,msg: String) {
                LogUtils.e(TAG, "fetchCoursewareNotice request error,code $code,msg:$msg")
            }

        }, this)
    }

    fun fetchDownloadUrl(coursewareId: String, listener: IUrlListener? = null) {
        if (coursewareId.isNullOrBlank()) {
            return
        }
        mUrlCall?.dispose()

        HttpModel.fetchDownloadUrl(coursewareId, object : JsonDataObserver() {
            override fun onSubscribe(d: Disposable) {
                mUrlCall = d
                LogUtils.i(TAG, "fetchDownloadUrl request start")
            }

            override fun onError(code:Int,msg: String) {
                LogUtils.e(TAG, "fetchDownloadUrl request error, code:$code msg:$msg")
                listener?.onError()
            }

            override fun onSuccessWithResult(result: JsonObject) {
                val url = GsonUtil.getString(result, "url")
                if (listener != null) {
                    if (url.isNullOrBlank()) listener.onError() else listener.onSuccess(url)
                }
                LogUtils.i(TAG, "fetchDownloadUrl success")
            }
        })
    }

    private fun getUpdateTitle(time: String): String {
        return SharedPrefsUtil.getString(redGetSPTable(), time + "str", "")
    }

    private fun redGetSPTable(): String? {
        return if (TextUtils.isEmpty(mCourseId)) null else String.format(SP_COURSE_WARE_RED_TABLE, mCourseId)
    }

    private fun redUpdate(time: String, num: String) {
        // 若还未初始化数据，获取一次历史数据
        if (TextUtils.isEmpty(mTime)) {
            mTime = redGetLatestTime()
            LogUtils.li(TAG, "getLatestTime,time:$mTime")
        }

        // 取到的数据没有历史数据新，不处理
        if (!TextUtils.isEmpty(mTime) && time.toLong() <= mTime!!.toLong()) {
            return
        }
        LogUtils.li(TAG, "clearRed,time:$mTime")
        // 清除之前的数据
        SharedPrefsUtil.getSharedPreferences(redGetSPTable()).edit().remove(mTime).apply()
        SharedPrefsUtil.getSharedPreferences(redGetSPTable()).edit().remove(mTime + "str").apply()
        mTime = time
        LogUtils.li(TAG, "updateRed,time:$mTime")
        SharedPrefsUtil.putBoolean(redGetSPTable(), mTime, true)
        SharedPrefsUtil.putSting(redGetSPTable(), mTime + "str", String.format(Locale.getDefault(), "老师于%s更新了%s个课件",
                DateUtil.formatTime(time.toLong(), "MM月dd日HH:mm"), num))
    }

    fun redIsNeedShow(): Boolean {
        if (TextUtils.isEmpty(mTime)) {
            mTime = redGetLatestTime()
            LogUtils.li(TAG, "getLatestTime,time:$mTime")
        }
        val b = !TextUtils.isEmpty(mTime) && SharedPrefsUtil.getBoolean(redGetSPTable(), mTime, false)
        LogUtils.li(TAG, "redIsNeedShow,time:$mTime,b:$b")
        return b
    }

    /**
     * 获取课程之前的时间
     */
    private fun redGetLatestTime(): String? {
        val sp = SharedPrefsUtil.getSharedPreferences(redGetSPTable())
        val map = sp.all
        if (map == null || map.isEmpty()) {
            return null
        }
        for (key in map.keys) {
            val value = map[key]
            if (value is Boolean) {
                return key
            }
        }
        return null
    }

    fun redClicked() {
        LogUtils.li(TAG, "clickedRed,time:$mTime")
        if (TextUtils.isEmpty(mTime)) {
            return
        }
        SharedPrefsUtil.putBoolean(redGetSPTable(), mTime, false)
    }

    fun registerLifecycle(owner: LifecycleOwner) {
        owner.lifecycle.addObserver(this)
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    fun onDestroy() {
        HttpModel.cancel(this)
    }

    fun setCourseId(courseId: String?) {
        mCourseId = courseId
    }

    interface INoticeListener {
        fun onResult(isNeedShow: Boolean, updateTitle: String?)
    }

    interface IUrlListener {
        fun onSuccess(url: String?)
        fun onError()
    }

    companion object {
        private const val SP_COURSE_WARE_RED_TABLE = "CoursewareRed-%s"
        private const val TAG = "CoursewareRequester"
    }
}