/*
 * Copyright © 1998 - 2020 Tencent. All Rights Reserved.
 * 腾讯公司 版权所有
 */
package com.tencent.edu.module.course.task.courseware

import android.content.Context
import android.util.Log
import android.view.MotionEvent
import android.view.View
import android.widget.FrameLayout
import android.widget.SeekBar
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleObserver
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.OnLifecycleEvent
import com.tencent.edu.R
import com.tencent.edu.arm.player.view.ARMVideoView
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.utils.StringUtil
import kotlinx.android.synthetic.main.layout_preview_audio.view.preview_container
import kotlinx.android.synthetic.main.layout_preview_audio.view.preview_ctrl
import kotlinx.android.synthetic.main.layout_preview_audio.view.preview_name
import kotlinx.android.synthetic.main.layout_preview_audio.view.preview_seek_bar
import kotlinx.android.synthetic.main.layout_preview_audio.view.preview_seek_time
import kotlinx.android.synthetic.main.layout_preview_audio.view.preview_size
import kotlinx.android.synthetic.main.layout_preview_video.view.*
import java.io.File
import java.util.*

/**
 * @author aidanyang
 * @date 2020-6-29
 */
class VideoPreviewView constructor(context: Context) : FrameLayout(context), LifecycleObserver {

    private var mTimer: Timer? = null
    private lateinit var mPlayer: ARMVideoView
    private var isPrepared = false
    private var isPause = false

    private var mTotalTime = 0L
    private var mLastTime = 0L

    private val autoHideRunnable = Runnable {
        preview_ctrl.visibility = View.GONE
    }

    init {
        initView()
        initPlayer()
    }

    private fun initPlayer() {
        mPlayer = preview_video
        mPlayer.setPlayProxy(MyLocalFilePlayCache())
        mPlayer.setOnPreparedListener {
            isPrepared = true

            mTotalTime = mPlayer.duration.toLong()
            refreshSeekTime(0)

            startPlay()
        }
        mPlayer.setOnCompletionListener {
            playComplete()
        }
    }

    private fun playComplete() {
        cancelAutoHideCtrl()
        preview_ctrl.setImageResource(R.drawable.ic_preview_play)
        preview_seek_ctrl.setImageResource(R.drawable.ic_preview_play_s)

        preview_ctrl.visibility = View.VISIBLE
        isPause = true

        stopTimer()

        refreshSeekTime(mTotalTime)
        preview_seek_bar.progress = MAX_PROGRESS
    }

    private fun refreshSeekTime(current: Long) {
        preview_seek_time.text = String.format("%s/%s",
                StringUtil.millisToStringShort(current),
                StringUtil.millisToStringShort(mTotalTime))
    }

    private fun initView() {
        View.inflate(context, R.layout.layout_preview_video, this)

        preview_seek_bar.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(seekBar: SeekBar?, progress: Int, fromUser: Boolean) {
                val msec = mPlayer.duration * progress / MAX_PROGRESS
                refreshSeekTime(msec.toLong())
            }

            override fun onStartTrackingTouch(seekBar: SeekBar?) {
                stopTimer()
            }

            override fun onStopTrackingTouch(seekBar: SeekBar?) {
                val msec = mPlayer.duration * preview_seek_bar.progress / MAX_PROGRESS
                mPlayer.seekTo(msec)

                if (mPlayer.currentPosition != mPlayer.duration) {
                    startPlay()
                    autoHideCtrl()
                } else {
                    playComplete()
                }
            }
        })

        val play = OnClickListener {
            when {
                mPlayer.isPlaying -> {
                    isPause = true
                    pausePlay()
                    cancelAutoHideCtrl()
                }
                else -> {
                    startPlay()
                    autoHideCtrl()
                }
            }
        }

        preview_ctrl.setOnClickListener(play)
        preview_seek_ctrl.setOnClickListener(play)

        preview_container.setOnTouchListener { _, event ->
            if (isPause) {
                return@setOnTouchListener true
            }

            if (event.action == MotionEvent.ACTION_DOWN) {
                if (preview_ctrl.visibility == View.GONE) {
                    preview_ctrl.visibility = View.VISIBLE
                } else {
                    preview_ctrl.visibility = View.GONE
                }

                autoHideCtrl()
            }
            true
        }
    }

    // 自动隐藏控制按钮
    private fun autoHideCtrl() {
        cancelAutoHideCtrl()
        postDelayed(autoHideRunnable, CTRL_AUTO_HIDE_DELAY)
    }

    private fun cancelAutoHideCtrl() {
        removeCallbacks(autoHideRunnable)
    }

    private fun startPlay() {
        if (!isPrepared) {
            return
        }

        preview_ctrl.setImageResource(R.drawable.ic_preview_pause)
        preview_seek_ctrl.setImageResource(R.drawable.ic_preview_pause_s)

        mLastTime = 0
        isPause = false
        startTimer()
        mPlayer.start()
    }

    private fun pausePlay() {
        preview_ctrl.setImageResource(R.drawable.ic_preview_play)
        preview_ctrl.visibility = View.VISIBLE
        preview_seek_ctrl.setImageResource(R.drawable.ic_preview_play_s)

        // isPause不放在这里因为考虑到要判断onResume时是否继续播放
        stopTimer()
        mPlayer.pause()
    }

    fun setFile(path: String) {
        val file = File(path)
        if (file.length() == 0L) {
            return
        }

        preview_name.text = file.name
        preview_size.text = StringUtil.getFileSizeString(file.length())
        mPlayer.setVideoPath(path)
        startPlay()
    }

    private fun startTimer() {
        mTimer = Timer()
        mTimer?.schedule(object : TimerTask() {
            override fun run() {
                ThreadMgr.getInstance().executeOnUiThread {
                    if (isPause){
                        return@executeOnUiThread
                    }

                    // 防止播放到最后突然回退的问题
                    if (mLastTime > mPlayer.currentPosition) {
                        return@executeOnUiThread
                    }

                    mLastTime = mPlayer.currentPosition.toLong()
                    refreshSeekTime(mPlayer.currentPosition.toLong())
                    preview_seek_bar.progress = (mPlayer.currentPosition.toFloat() / mPlayer.duration * 100).toInt()
                }
            }
        }, TIMER_PERIOD, TIMER_PERIOD)
    }

    private fun stopTimer() {
        mTimer?.cancel()
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_RESUME)
    fun onResume(owner: LifecycleOwner) {
        Log.e(TAG, "onResume")

        if (isPrepared && !isPause) {
            startPlay()
        }
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_PAUSE)
    fun onPause(owner: LifecycleOwner) {
        Log.e(TAG, "onPause")

        if (isPrepared && mPlayer.isPlaying) {
            pausePlay()
        }
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    fun onDestroy(owner: LifecycleOwner) {
        Log.e(TAG, "onDestroy")

        mPlayer.releaseWithoutStop()
        mTimer?.cancel()
    }

    companion object {

        private const val TAG = "VideoPreviewView"

        // 3s后隐藏控制按钮
        private const val CTRL_AUTO_HIDE_DELAY = 3000L

        private const val MAX_PROGRESS = 100

        // 更新SeekBar的时间
        private const val TIMER_PERIOD = 100L

    }
}