package com.tencent.edu.module.homepage.data

import com.google.gson.Gson
import com.google.gson.JsonObject
import com.tencent.edu.common.event.EventMgr
import com.tencent.edu.common.misc.LocalUri
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.SharedPrefsUtil
import com.tencent.edu.common.utils.VersionUtils
import com.tencent.edu.framework.EduFramework
import com.tencent.edu.kernel.AppRunTime
import com.tencent.edu.kernel.KernelEvent
import com.tencent.edu.kernel.KernelUtil
import com.tencent.edu.kernel.http.HttpModel
import com.tencent.edu.kernel.http.data.JsonDataObserver
import com.tencent.edu.kernel.http.data.ObjectDataObserver
import com.tencent.edu.kernel.login.mgr.LoginMgr
import com.tencent.edu.module.homepage.model.EnjoyStudyCardDetail
import com.tencent.edu.module.homepage.model.EnjoyStudyCardInfo
import com.tencent.edu.module.homepage.newhome.mine.IMyHomePageDataEvt
import com.tencent.edu.module.welfare.EnjoyStudyCardCenter
import io.reactivex.disposables.Disposable

/**
 * 畅学卡请求类
 */
object EnjoyStudyCardRequester {

    private const val TAG = "EnjoyStudyCardRequester"

    private const val KEY_RED_DOT = "Red"

    //    800032867 (主站)
//    800030907 (百度SEM)
//    800032717（搜狗SEM）
//    800028859（头条）
//    800031867（广点通）
//    800032289（微信广告）
    private val ENJOY_STUDY_CARD_CHANNEL = listOf("800032867", "800030907", "800032717", "800028859", "800031867", "800032289")

    private var mInfoDispose: Disposable? = null
    private var mDetailDispose: Disposable? = null
    private var isFirstRequest = true
    private var isAppFirstRun: Boolean? = null
    private var isChannelFirstRun: Boolean? = null

    var mInfo: EnjoyStudyCardInfo? = null
    var mDetail: EnjoyStudyCardDetail? = null

    private fun getRedKey(): String {
        return KEY_RED_DOT + KernelUtil.getAssetAccountId()
    }

    fun grantOrGetStudyCard(mEvt: List<IMyHomePageDataEvt>) {
        // 非登录态不请求接口
        if (!LoginMgr.getInstance().isLogin) {
            for (it in mEvt) {
                it.onEnjoyStudyReqComplete(mInfo)
            }
            return
        }

        mInfoDispose?.dispose()

        if (isEnjoyStudyCardChannel()) {
            // 判断是否畅学卡渠道，如果是，直接跳转
            if (isChannelFirstRun()) {
                jumpToWebPage(true)
                return
            }
        } else if (isAppFirstRun() && isFirstRequest) {
            isFirstRequest = false
            LogUtils.i(TAG, "is app first run")
            doGetStudyCardGrantInfo(mEvt)
            return
        }

        doGrantOrGetStudyCardRequest(mEvt)
    }

    private fun isEnjoyStudyCardChannel(): Boolean {
        return ENJOY_STUDY_CARD_CHANNEL.contains(VersionUtils.getChannelIdFromIni(EduFramework.getApplicationContext()))
    }

    private fun doGetStudyCardGrantInfo(mEvt: List<IMyHomePageDataEvt>) {
        HttpModel.getStudyCardGrantInfo(object : JsonDataObserver() {

            override fun onSubscribe(d: Disposable) {
                mInfoDispose = d
                LogUtils.i(TAG, "getStudyCardGrantInfo request start")
            }

            override fun onError(e: String) {
                LogUtils.e(TAG, "getStudyCardGrantInfo request error")
                doGrantOrGetStudyCardRequest(mEvt)
            }

            override fun onSuccessWithResult(result: JsonObject) {
                mInfo = Gson().fromJson(result, EnjoyStudyCardInfo::class.java)
                if (mInfo?.isGranted() == true) {
                    LogUtils.i(TAG, "is app first run and is granted")

                    jumpToWebPage()
                    EventMgr.getInstance().notify(KernelEvent.EVENT_NOTIFY_ENJOY_STUDY_CARD_INFO, mInfo)
                } else {
                    LogUtils.i(TAG, "is app first run and is not granted")

                    doGrantOrGetStudyCardRequest(mEvt)
                }

                LogUtils.e(TAG, "getStudyCardGrantInfo request success")
            }

        })
    }

    private fun doGrantOrGetStudyCardRequest(mEvt: List<IMyHomePageDataEvt>) {
        HttpModel.grantOrGetStudyCard(if (AppRunTime.getInstance().currentTabPosition == 0) 1 else 0, object : JsonDataObserver() {

            override fun onSubscribe(d: Disposable) {
                mInfoDispose = d
                LogUtils.i(TAG, "fetchEnjoyStudyInfo request start")
            }

            override fun onError(code: Int, msg: String) {
                LogUtils.e(TAG, "fetchEnjoyStudyInfo request error")
            }

            override fun onSuccessWithResult(result: JsonObject) {
                mInfo = Gson().fromJson(result, EnjoyStudyCardInfo::class.java)
                LogUtils.i(TAG, "fetchEnjoyStudyInfo request success")
            }

            override fun onComplete() {
                // 回调表示已完成请求
                for (it in mEvt) {
                    it.onEnjoyStudyReqComplete(mInfo)
                }

                EventMgr.getInstance().notify(KernelEvent.EVENT_NOTIFY_ENJOY_STUDY_CARD_INFO, mInfo)
            }

        })
    }

    fun isShowRedDot(): Boolean {
        return SharedPrefsUtil.getBoolean(
                EnjoyStudyCardCenter.SP_TABLE,
                getRedKey(),
                true
        )
    }

    fun clickRedDot() {
        SharedPrefsUtil.putBoolean(
                EnjoyStudyCardCenter.SP_TABLE,
                getRedKey(),
                false
        )
    }

    fun getStudyCourseDetail(cid: String, termId: String) {
        mDetailDispose?.dispose()

        HttpModel.getStudyCourseDetail(termId, object : ObjectDataObserver<EnjoyStudyCardDetail>() {

            override fun onSubscribe(d: Disposable) {
                mDetailDispose = d
                LogUtils.i(TAG, "getStudyCourseDetail request start")
            }

            override fun onSuccessWithResult(result: EnjoyStudyCardDetail) {
                mDetail = result
                mDetail!!.termId = termId
                mDetail!!.cid = cid
                LogUtils.i(TAG, "getStudyCourseDetail request success,$result")
            }

            override fun onError(msg: String) {
                LogUtils.e(TAG, "getStudyCourseDetail request error")
            }

            override fun onComplete() {
                notifyDetailResult()
            }

        })
    }

    fun syncCanGrantStudyCard(termId: String) {
        HttpModel.syncCanGrantStudyCard(termId, object : JsonDataObserver() {

            override fun onSubscribe(d: Disposable) {
                LogUtils.i(TAG, "syncCanGrantStudyCard request start")
            }

            override fun onSuccess(jsonResp: JsonObject): Boolean {
                LogUtils.i(TAG, "syncCanGrantStudyCard request success")
                mDetail?.is_granted = 1
                return true
            }

            override fun onError(code: Int, msg: String) {
                LogUtils.e(TAG, "syncCanGrantStudyCard request error,$msg")
            }
        })
    }

    private fun notifyDetailResult() {
        EventMgr.getInstance().notify(KernelEvent.EVENT_NOTIFY_ENJOY_STUDY_CARD_DETAIL, mDetail)
    }

    @JvmOverloads
    fun jumpToWebPage(force: Boolean = false) {
        LocalUri.jumpToEduUri(
                "tencentedu://openpage/enjoy_study_card?theme=1&share=0&" +
                        "url=https://m.ke.qq.com/m-core/freeLearningList.html" +
                        if (force)
                            "?isGrant=1??logic_no=1" // 这里??用于表示&,到webView里loadUrl时替换
                        else
                            ""
        )
    }

    private fun isChannelFirstRun(): Boolean {
        if (isChannelFirstRun != null) {
            return false
        }
        isChannelFirstRun = SharedPrefsUtil.getBoolean(TAG, "ChannelFirstRun", true)
        SharedPrefsUtil.putBoolean(TAG, "ChannelFirstRun", false)
        return isChannelFirstRun!!
    }

    private fun isAppFirstRun(): Boolean {
        if (isAppFirstRun != null) {
            return isAppFirstRun!!
        }
        isAppFirstRun = SharedPrefsUtil.getBoolean(TAG, "FirstRun", true)
        SharedPrefsUtil.putBoolean(TAG, "FirstRun", false)
        return isAppFirstRun!!
    }

}