package com.tencent.edu.module.shortvideo.playerview

import android.app.Activity
import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.animation.AlphaAnimation
import android.widget.LinearLayout
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleObserver
import androidx.lifecycle.OnLifecycleEvent
import com.tencent.edu.R
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.utils.MiscUtils
import com.tencent.edu.module.shortvideo.bean.VideoBean
import com.tencent.edu.module.vodplayer.player.EduARMVideoView
import com.tencent.edu.module.vodplayer.widget.PlayerScreenLockView
import kotlinx.android.synthetic.main.actionbar_base_custom.view.*
import kotlinx.android.synthetic.main.layout_simple_player_control_view_landscape.view.*

/**
 * 短视频横屏控制面板
 *  ActionBar
 *  LockView
 *  ControllerView
 *
 * @author aidanyang
 * @date 2020-12-9
 */
class ShortVideoLandscapeActionView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) : LinearLayout(context, attrs, defStyleAttr), LifecycleObserver {

    companion object {
        private const val TAG = "ShortVideoLandscapeActionView"
        private const val SHOW_DURATION = 3000L
    }

    private lateinit var mShowAction: AlphaAnimation
    private lateinit var mHiddenAction: AlphaAnimation
    private var isShow = false

    private var mLockListener: IOnLockListener? = null

    private val mHideRunnable = Runnable { showWithAnimation(false) }

    init {
        initLayout()
        initListener()
        initAnimation()
    }

    // 主要是刷新控制面板的显示时间
    private fun initListener() {
        short_video_lock_view.setIScreenLockListener(object : PlayerScreenLockView.IScreenLockListener {
            override fun onLockScreenShow(lock: Boolean) {
                mLockListener?.onLock(lock)
                // 刷新控制面板的显示时间
                autoHide(true)

                // 如果锁定，除了LockView之外的View隐藏
                showWithAnimationExcludeLockView(!lock)
            }
        })

        short_video_controller.setIsAutoHideCallback { autoHide(it) }
    }

    private fun showWithAnimationExcludeLockView(show: Boolean) {
        val animation = if (show) mShowAction else mHiddenAction
        short_video_controller.showWithAnimation(show, animation)
        short_video_action_bar.showWithAnimation(show, animation)
    }

    private fun initLayout() {
        LayoutInflater.from(context).inflate(R.layout.layout_simple_player_control_view_landscape, this, true)
        short_video_action_bar.action_bar_back.setOnClickListener {
            MiscUtils.switchOrientation(false, context as Activity, false)
        }
        player_gesture.setPreviewLayer(short_video_shadow_layer)
    }

    private fun initAnimation() {
        mShowAction = AlphaAnimation(0.0f, 1.0f)
        mShowAction.duration = 300
        mHiddenAction = AlphaAnimation(1.0f, 0.0f)
        mHiddenAction.duration = 300
    }

    fun uninit() {
        short_video_controller.uninit()
        ThreadMgr.removeUIRunnable(mHideRunnable)
    }

    /**
     * 显示/隐藏控制面板（带动画）
     */
    fun showWithAnimation(forceShow: Boolean) {
        isShow = forceShow || !isShow
        autoHide(isShow)
        val animation = if (isShow) mShowAction else mHiddenAction

        short_video_lock_view.showWithAnimation(isShow, animation)
        if (isShow && short_video_lock_view.isLockState) {
            return
        }

        showWithAnimationExcludeLockView(isShow)
    }

    /**
     * 显示/隐藏控制面板
     */
    fun show(forceShow: Boolean = false) {
        isShow = forceShow || !isShow
        autoHide(isShow)

        short_video_lock_view.show(isShow)
        if (isShow && short_video_lock_view.isLockState) {
            return
        }

        short_video_controller.show(isShow)
        short_video_action_bar.show(isShow)
    }

    private fun autoHide(show: Boolean) {
        ThreadMgr.removeUIRunnable(mHideRunnable)
        if (show) {
            ThreadMgr.postToUIThread(mHideRunnable, SHOW_DURATION)
        }
    }

    fun isLockState(): Boolean {
        return short_video_lock_view.isLockState
    }

    override fun setVisibility(visibility: Int) {
        super.setVisibility(visibility)
        show(visibility == View.VISIBLE)
    }

    fun setMediaPlayer(player: EduARMVideoView) {
        short_video_controller.setMediaPlayer(player)
        player_gesture.setMediaPlayer(player)
    }

    fun setVideoDetail(data: VideoBean) {
        short_video_action_bar.setVideoDetail(data)
        short_video_controller.setVideoDetail(data)
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    fun onDestroy() {
        uninit()
    }

    fun onProgressSlide(distanceX: Float) {
        player_gesture.onProgressSlide(distanceX)
    }

    fun onVolumeSlide(distanceX: Float) {
        player_gesture.onVolumeSlide(distanceX)
    }

    fun onBrightnessSlide(distanceX: Float) {
        player_gesture.onBrightnessSlide(distanceX)
    }

    fun endGesture() {
        player_gesture.endGesture()
    }

    fun onVideoScaleChange(xOffset: Int, yOffset: Int, scale: Float) {
        player_gesture.onVideoScaleChange(xOffset, yOffset, scale)
    }

    fun setPipBtnListener(l: OnClickListener) {
        short_video_action_bar.setPipBtnListener(l)
    }

    fun setOnLockListener(l: IOnLockListener) {
        mLockListener = l
    }

    fun hideTips(){
        short_video_controller.hideTips()
    }

    interface IOnLockListener {
        fun onLock(lock: Boolean)
    }

}